from random import randint

def печатать(текст, стиль="", красный=False, фиолетовый=False, зеленый=False, оранжевый=False, голубой=False, желтый=False, не_печатать=False, рандомный=False):
    def определить_стиль(стиль: str):
        стиль = стиль.lower()
        результат = ""
        if "ж" in стиль:
            результат += ";1"
        elif "к" in стиль:
            результат += ";3"
        elif "ч" in стиль:
            результат += ";4"
        elif "з" in стиль:
            результат += ";9"
        return результат
    
    итоговый_стиль = определить_стиль(стиль)
    if красный:
        цвет = "225;0;0" 
    elif фиолетовый:
        цвет = "0;0;255"
    elif зеленый:
        цвет = "0;128;0"
    elif оранжевый:
        цвет = "255;127;0"
    elif голубой:
        цвет = "28;170;214"
    elif желтый:
        цвет = "255;255;0"

    elif рандомный:
        результат = ""
        for буква in текст:
            к = randint(0,225)
            з = randint(0, 225)
            с = randint(0,225)
            результат += f"\x1B[38;2;{к};{з};{с}{итоговый_стиль}m{буква}"
        результат += "\x1B[0m"
        print(результат)
        return

    else:
        цвет = "255;255;255"
    результат = f"\x1B[38;2;{цвет}{итоговый_стиль}m{текст}\x1B[0m"
    if не_печатать:
        return результат
    print(результат)
    # \x1B - особый символ, который включает режим друого цвета
    # 38;2; - говорит о том, что следующие 3 команды будут в формате RGB
    # ;1 - особый стиль (в данном случае жирный)
    # \x1B[0m - всё, что будет написано после этого символа будет стндартным цветом. Еси не будет этой строчки, все последующие print будут писаться одним и тем же цветом


if __name__ == "__main__":
    красный = печатать("Красный текст", красный=True, не_печатать=True)
    зелёный = печатать("зелёный текст", зеленый=True, не_печатать=True)
    print(f"{красный} и {зелёный} в одной строчке")


from .terminal import печатать
print(" ")

def содержит(что, где, игнорировать_регистр=False):
    # функция проверяет, что хотя бы один элемент из "что" (или вся переменная целиком) содержится хотя бы в одном элементе из "где" (или является частью самой переменной "где")

    # функция возвращает True или False
    # переменные "что" и "где" могут быть двух видов:
        # 1. итерируемые объекты (кортеж или список), всеми элементами которых являются строки (текст)
        # 2. сама переменная может быть строкой (текстом)
    # при нарушении любого из этих пунктов будет вызвана ошибка с описанием проблемы
    # переменная "игнорировать_регистр" может принимать только True, или быть по умолчанию False. При передаче другого типа данных будет вызвана ошибка.
    # при получении в "что" или "где" пустых строк или пустых списков будет возвращаться False
    if type(что) not in [str, list, tuple]:
        ошибка = f"переменная \"что\" может быть только текстом, списком или кортежом. {type(что)}"
        raise TypeError(печатать(ошибка, красный=True, не_печатать=True))

    if type(где) not in [str, list, tuple]:
        ошибка = f"переменная \"где\" может быть только текстом, списком или кортежом. {type(где)}"
        raise TypeError(печатать(ошибка, красный=True, не_печатать=True))
    
    if type(игнорировать_регистр) != bool:
        ошибка = f"переменная \"где\" может быть только True или False. {type(игнорировать_регистр)}"
        raise TypeError(печатать(ошибка, красный=True, не_печатать=True))
    
    if type(что) != str:
        for элемент in что:
            if type(элемент) != str:
                ошибка = f"В переменной \"что\" оказался нетекстовый тип - {элемент}"
                raise TypeError(печатать(ошибка, красный=True, не_печатать=True))
    
    if type(где) != str:
        for элемент in где:
            if type(элемент) != str:
                ошибка = f"В переменной \"где\" оказался нетекстовый тип - {элемент}"
                raise TypeError(печатать(ошибка, красный=True, не_печатать=True))

    if игнорировать_регистр:
        if type(что) == str:
            что = что.lower()
        
        else:
            новый_список = []
            for элемент in что:
                новый_список.append(элемент.lower())
            что = новый_список


        if type(где) == str:
            где = где.lower()
        
        else:
            новый_список = []
            for элемент in где:
                новый_список.append(элемент.lower())
            где = новый_список
    
    if type(что) == type(где) == str:
        if что in где:
            return True
        
    if type(что) == str and type(где) != str:
        for элемент in где:
            if что in элемент:
                return True
    
    if type(что) != str and type(где) == str:
        for элемент in что:
            if элемент in где:
                return True
    
    if type(что) != str and type(где) != str:
        for элемент_что in что:
            for элемент_где in где:
                if элемент_что in элемент_где:
                    return True






    return False

def тесты():
    # TDD - Test Driven Development
    # Разработка функций после написания тестов к ней

    # Согласно этому принципу, нужно сначала написать тесты, которые проверят аспект работы каждой функции. Каждый рвз при разработке функции необходимо запускать тесты - если они все пройдены, значит функция точно работает правильно

    # При разработке тестов недопустимо использоване вычесдяемых переменных - тестируемая функция должна принять заранее подготовленные данные, а тест проверяет, согласуется ли её ответ с ожидаемым.
    # Например при тестировании функции "сумма" ей в аргументы передаются числа 15 и 10, исравнивается её ответ с числом 25, а не с вычислением 15 + 10.
    
    пройдено = []
    не_пройдено = []

    def тест(что, где, игнорировать_регистр, ожидаемый_ответ, название_теста):
        try:
            ответ = содержит(что, где, игнорировать_регистр)
            ошибка = None
        except Exception as error:
            ошибка = error
            ответ = "ошибка"

        if ответ == ожидаемый_ответ:
            пройдено.append(1)
            печатать(название_теста, зеленый=True)
        else:
            печатать(название_теста, красный=True)
            print(f"ожидаемый - {ожидаемый_ответ}")
            print(f"ответ     - {ответ}")
            if ошибка:
                print(ошибка)
            print()
            не_пройдено.append(0)
    
    тест(123, "где", True, "ошибка", "неправильный тип - что")
    тест("что", 123, True, "ошибка", "неправильный тип - где")
    тест("что", "где", "текст", "ошибка", "неправильный тип - игнорировать регистр")
    тест(["текст", "не цифра", 42], "где", False, "ошибка", "тип \"что\" типа str")
    тест(
        что="что",
        где=["текст", "не цифра", 42],
        игнорировать_регистр=False,
        ожидаемый_ответ="ошибка",
        название_теста='Переменной \"где\" типа не str'
        )
    
    варианты = [
        ["сырники", "кто-то ест Сырники", True, True, "что - строка; где - строка; игн_регистр - True; ответ - True"],
        ["сырники", "кто-то ест грибы", True, False, "что - строка; где - строка; игн_регистр - True; ответ - False"],
        ["Сырники", "кто-то ест Сырники", False, True, "что - строка; где - строка; игн_регистр - False; ответ - True"],
        ["сырники", "кто-то ест Сырники", False, False, "что - строка; где - строка; игн_регистр - False; ответ - False"],
        ["Сырники", ["кто-то ест Сырники", "кто-то ест грибы"], False, True, "что - строка; где - список; игн_регистр - False; ответ - True"],
        ["сырники", ["кто-то ест Сырники", "кто-то ест грибы"], False, False, "что - строка; где - список; игн_регистр - False; ответ - False"],
        ["сырники", ["кто-то ест Сырники", "кто-то ест грибы"], True, True, "что - строка; где - список; игн_регистр - True; ответ - True"],
        ["пряники", ["кто-то ест Сырники", "кто-то ест грибы"], True, False, "что - строка; где - список; игн_регистр - True; ответ - False"],
        [["Грибы", "Сырники"], "Кто-то ест Грибы", False, True, "что - список; где - строка; игн_регистр - False; ответ - True"],
        [["Грибы", "Сырники"], "Кто-то ест грибы", False, False, "что - список; где - строка; игн_регистр - False; ответ - False"],
        [["Грибы", "Сырники"], "Кто-то ест грибы", True, True, "что - список; где - строка; игн_регистр - True; ответ - True"],
        [["Грибы", "Сырники"], "Кто-то ест пряники", True, False, "что - список; где - строка; игн_регистр - True; ответ - False"],
        [["Грибы", "Сырники"], ["кто-то ест Сырники", "кто-то ест Грибы"], False, True, "что - список; где - список; игн_регистр - False; ответ - True"],
        [["Грибы", "Сырники"], ["кто-то ест сырники", "кто-то ест грибы"], False, False, "что - список; где - список; игн_регистр - False; ответ - False"],
        [["Грибы", "Сырники"], ["кто-то ест сырники", "кто-то ест грибы"], True, True, "что - список; где - список; игн_регистр - True; ответ - True"],
        [["Грибы", "Сырники"], ["кто-то ест пряники", "кто-то ест пельмени"], True, False, "что - список; где - список; игн_регистр - True; ответ - False"],
    ]

    for вариант in варианты:
        что, где, игнорировать_регистр, ответ, название = вариант
        тест(что, где, игнорировать_регистр, ответ, название)
    
    print(f"---------------------------------\nВсего тестов - {len(пройдено + не_пройдено)}\nПройдено - {len(пройдено)}\nНе пройдено - {len(не_пройдено)}")

тесты()


РАЗДЕЛИТЕЛЬ_ЗНАЧЕНИЙ = "_____++_*$$#!!~#@$$^&*(("
РАЗДЕЛИТЕЛЬ_ПАР = "vbhj xc78677@#$%^&*)()"
def write_file(text: str, path: str):
    try:
        with open(path, mode="w", encoding="utf-8") as file:
            file.write(text)
    except Exception as write_er:
        print(f"Произошла ошибка \"{write_er}\"")
        return False
    return True

def read_file(path: str):
    try:
        with open(path, mode="r", encoding="utf-8") as file:
            return file.read()
    except Exception as read_er:
        print(f"Произошла ошибка \"{read_er}\"")
        return False
    return True

def словарь_в_текст(словарь: dict):
    результат = ""
    for пара in словарь.items():
        ключ, значение = пара
        текстовая_пара = f"{ключ}{РАЗДЕЛИТЕЛЬ_ЗНАЧЕНИЙ}{значение}" # type: ignore
        результат += текстовая_пара + РАЗДЕЛИТЕЛЬ_ПАР # type: ignore
    return результат[:-len(РАЗДЕЛИТЕЛЬ_ПАР)] # type: ignore

def текст_в_словарь(текст: str):
    результат = {}
    пары = текст.split(РАЗДЕЛИТЕЛЬ_ПАР) # type: ignore
    for пара in пары:
        ключ, значение = пара.split(РАЗДЕЛИТЕЛЬ_ЗНАЧЕНИЙ, 1) # type: ignore
        результат[ключ] = значение
    return результат

def преобразовать_все_словари(список_словарей: list):
    текст = ""
    for словарь in список_словарей:
        текст += словарь_в_текст(словарь) + "\n"
    return текст

def текст_в_словари(текст: str):
    список = []
    for строка in текст.splitlines():
        словарь = текст_в_словарь(строка)
        список.append(словарь)
    return список

if __name__ == "__main__":
    список_словарей = [{"ключ1": "значение1", "ключ2": "значение2"},{"ключ3": "значение3", "ключ4": "значение4"}]
    текст = преобразовать_все_словари(список_словарей)
    обратно = текст_в_словари(текст)
    print(список_словарей == обратно)


from setuptools import setup

setup(
    name="coloured_terminal", # Имя модуля, которое будет использоваться через pip install
    version="0.2", # версия модуля. При каждом обновлении её нужно увеличивать
    description="Писать цветными буквами в терминале", # Описание проекта, его можно не использовать
    packages=["coloured-terminal"], # Список с названиями папок, которые нужно залить
    author_email="nikitastrukalin747@icloud.com", # Почта автора
    zip_safe=False, # Пароль, можно не указываьть
    install_requires = [], # Список дополнительных модулей, которые необходимо использовать для вашего модуля
)